------------------------------
--      Are you local?      --
------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Postal")

---------------------------------
--      Addon Declaration      --
---------------------------------

Postal = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceModuleCore-2.0", "AceConsole-2.0", "AceDB-2.0", "AceHook-2.1")

Postal.revision = tonumber(string.sub("$Revision: 40510 $", 12, -3))

Postal:SetModuleMixins("AceEvent-2.0", "AceHook-2.1")

Postal.defaults = {}

Postal.consoleOptions = {
	type = "group",
	handler = Postal,
	args = {},
}

Postal:RegisterDB("PostalDB", "PostalDBPerChar")
Postal:RegisterDefaults("profile", Postal.defaults)
Postal:RegisterChatCommand({ "/Postal" }, Postal.consoleOptions, "POSTAL")

------------------------------
--      Initialization      --
------------------------------

function Postal:OnInitialize()
	if not self.version then self.version = GetAddOnMetadata("Postal", "Version") end
	local rev, name = self.revision, nil
	for name, module in self:IterateModules() do
		rev = math.max(rev, module.revision)
	end
	self.version = (self.version or "1.0") .. " |cffff8888r" .. rev .. "|r"
end

function Postal:OnDisable()
	local name, module = nil, nil
	for name, module in self:IterateModules() do
		self:ToggleModuleActive(module, false)
	end
end

--------------------------------
--      Module Prototype      --
--------------------------------

Postal.modulePrototype.core = Postal
Postal.modulePrototype.revision = 1

function Postal.modulePrototype:OnInitialize()
	-- Unconditionally register, this shouldn't happen from any other place
	-- anyway.
	self.core:RegisterModule(self.name, self)
end

function Postal.modulePrototype:IsRegistered()
	return self.registered
end

-------------------------------
--      Module Handling      --
-------------------------------

function Postal:RegisterModule(name, module)
	if module:IsRegistered() then
		error(string.format("%q is already registered.", name))
		return
	end

	if module.db and module.RegisterDefaults and type(module.RegisterDefaults) == "function" then
		module:RegisterDefaults("char", module.defaults or {})
	else
		self:RegisterDefaults(name, "char", module.defaults or {})
	end

	if not module.db then module.db = self:AcquireDBNamespace(name) end

	if module.consoleOptions then
		local m = module.consoleCmd or name
		-- if the consoleoption already exists we merge in the data otherwise we create a new option
		if self.consoleOptions.args[m] then
			for k,v in pairs(module.consoleOptions.args) do self.consoleOptions.args[m].args[k] = v end
		else
			self.consoleOptions.args[m] = module.consoleOptions
		end
	end

	module.registered = true
	if module.OnRegister and type(module.OnRegister) == "function" then
		module:OnRegister()
	end
end

-------------------------------
--      Utility Functions    --
-------------------------------

-- Disable Inbox Clicks
function Postal:DisableInbox(disable)
	if disable then
		if not self:IsHooked("InboxFrame_OnClick") then
			self:Hook("InboxFrame_OnClick", function() end, true)
			for i = 1, 7 do
				_G["MailItem" .. i .. "ButtonIcon"]:SetDesaturated(1)
			end
		end
	else
		if self:IsHooked("InboxFrame_OnClick") then
			self:Unhook("InboxFrame_OnClick")
			for i = 1, 7 do
				_G["MailItem" .. i .. "ButtonIcon"]:SetDesaturated(nil)
			end
		end
	end
end

-- Is there a free slot in my bags?
function Postal:SpaceCheck()
	for bag = 0, 4 do
		local size = GetContainerNumSlots(bag)
		if size ~= nil and size > 0 then
			for slot = 1, size do
				if not GetContainerItemInfo(bag, slot) then return true end
			end
		end
	end
end

-- based on a free function at wowwiki
function Postal:tremovebyval(tab, val)
	for k,v in pairs(tab) do
		if v == val then
			table.remove(tab, k)
			return true
		end
	end
	return false
end
